@@ -1218,7 +1218,9 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri,
 	gint fd;
 	gint retval;
 	GeditSaveEncodingSetting encoding_setting;
-	
+	gboolean res;
+	gboolean add_cr;
+
 	gedit_debug (DEBUG_DOCUMENT, "");
 
 	g_return_val_if_fail (doc != NULL, FALSE);
@@ -1277,20 +1279,7 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri,
 	if (!real_filename)
 		goto out;
 
-	if (stat (real_filename, &st) != 0)
-	{
-		/* File does not exist? */
-		create_backup_copy = FALSE;
-
-		/* Use default permissions */
-		st.st_mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
-		st.st_uid = getuid ();
-		st.st_gid = getgid ();
-	}
-
-	/* Save to a temporary file.  We set the umask because some (buggy)
-	 * implementations of mkstemp() use permissions 0666 and we want 0600.
-	 */
+	/* Get the directory in which the real filename lives */
 
 	slashpos = strrchr (real_filename, G_DIR_SEPARATOR);
 
@@ -1302,6 +1291,36 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri,
 	else
 		dirname = g_strdup (".");
 
+	/* If there is not an existing file with that name, compute the
+	 * permissions and uid/gid that we will use for the newly-created file.
+	 */
+
+	if (stat (real_filename, &st) != 0)
+	{
+		struct stat dir_st;
+		int result;
+
+		/* File does not exist? */
+		create_backup_copy = FALSE;
+
+		/* Use default permissions */
+		saved_umask = umask (0);
+		st.st_mode = 0666 & ~saved_umask;
+		umask (saved_umask);
+		st.st_uid = getuid ();
+
+		result = stat (dirname, &dir_st);
+
+		if (result == 0 && (dir_st.st_mode & S_ISGID))
+			st.st_gid = dir_st.st_gid;
+		else
+			st.st_gid = getgid ();
+	}
+
+	/* Save to a temporary file.  We set the umask because some (buggy)
+	 * implementations of mkstemp() use permissions 0666 and we want 0600.
+	 */
+
 	temp_filename = g_build_filename (dirname, ".gedit-save-XXXXXX", NULL);
 	g_free (dirname);
 
@@ -1372,7 +1391,17 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri,
 	}	    
 
 	chars_len = strlen (chars);
-	if (write (fd, chars, chars_len) != chars_len)
+
+	add_cr = (*(chars + chars_len - 1) != '\n');
+
+	/* Save the file content */
+	res = (write (fd, chars, chars_len) == chars_len);
+
+	if (res && add_cr)
+		/* Add \n if needed */
+		res = (write (fd, "\n", 1) == 1);
+
+	if (!res)
 	{
 		gchar *msg;
 
@@ -1438,7 +1467,7 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri,
 		saved_errno = errno;
 
 		if (create_backup_copy && rename (backup_filename, real_filename) != 0)
-			g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, saved_errno,
+			g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, errno,
 				     " ");
 		else
 			g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, saved_errno,
